﻿************************************************************************************************************************************************************************************************
/PLEASE READ THE FOLLOWING MESSAGES VERY CAREFULLY BEFORE PROCEEDING TO USE GUMSCORE SPSS PROGRAM
/
/The GUMSCORE_SPSS macro conducts fixed-item parameter scoring for the Roberts' (2000) Generalized Graded Unfolding Model (GGUM) using
/Expected a Posteriori (EAP) scoring. This makes it possible to score a small group of persons (even only one individual) according to the GGUM. To do so,
/you must have: a) A set of item parameters for the measure from previous item parameter calibrations; and b) a dataset of people you wish to score
/according to those item parameters. GUMSCORE_SPSS can accomdate any number of persons, up to 100 items, with up to 9 response categories.Response 
/data must be pasted into the 'C:\GUMSCORE\Resp_File' and variable names should not be changed. Additionally, item parameters should be pasted into
/the 'C:\GUMSCORE\GGUM_ItemPars' file. All file locations and names must remain the same. Additionally the GUMSCORE folder should be saved directly
/to the C:\ drive.
/*NOTE: WARNING MESSAGES IN SPSS OUTPUT (.SPO) ARE TO BE EXPECTED. THEY ARE NOT AN INDICATION OF PROBLEMS WITH THE 
/*PROGRAM. /
/*NOTE: When creating the GGUM_ItemPars file, the variable t_0 should always be equal to 0 for all items. /
/*NOTE: GUMSCORE cannot score persons with missing item responses; if missing responses are included, that person will have a missing value for their /
/*theta estimate and standard error. A patch for this issue is currently being written. /
/*NOTE: No existing files should be changed or deleted, including the "COMPS" folder and the associated "Quadratures file; these should not be changed /
/*The GUMSCORE person may be used freely for all non-profit research and practice. For-profit practice should obtain direct permission from the authors for use./
/*
/*Please cite as: Carter, N.T., & LoPilato, A.C. (2014). GUMSCORE_SPSS: A SPSS Macro for Fixed-Item-Parameter Scoring for the Generalized Graded /
/*Unfolding Model [software program]. Athens, GA: University of Georgia /.
************************************************************************************************************************************************************************************************
************************************************************************************************************************************************************************************************
INSTRUCTIONS: To run GUMSCORE, the user needs to specify four pieces of information on the first line, beginning with !gumscore.
************************************************************************************************************************************************************************************************
1. C should be specified as the highest level of agreement. GUMSCORE assumes your responses are coded with 0 as the lowest option. Therefore, C is
the total number of response categories minus one. For example, when using a 6-point Likert-type scale, C=5 
2. i should be specified as the number of items in your scale. For example, when using a 10-item measure, i=10 
3. K should be specified as the total number of response categories. For example, for a 6-point Likert-type scale K=6 
4. p should be specified as the number of people to be scored (i.e., the number of respondents in th Resp_File.sav file). For example, if 25 persons need
to be scored, p=25 
************************************************************************************************************************************************************************************************

!gumscore C=5 / i=9 / K=6 / p=4.

DEFINE !gumscore (C = !CHAREND('/') / i = !CHAREND('/') / p = !CHAREND('/') / K = !CHAREND('/')). 
EXECUTE.
GET FILE='C:\GUMSCORE\GGUM_ItemPars.SAV'.
COMPUTE C=!C.
COMPUTE K=!K.
EXECUTE.
VECTOR t_=t_1 to t_8.
LOOP j=1 to !K.
VECTOR st_(!K).
END LOOP.
DO IF !K>1.
END IF.
LOOP j=1 to !K.
COMPUTE st_(j)=sum(st_(j-1),(t_(j))).
END LOOP.
EXECUTE.
DELETE VARIABLES t_1 to j.
EXECUTE.
MATRIX.
GET a / VARIABLES a.
COMPUTE ai=transpos(a).
MSAVE ai 
   / TYPE=MEAN
   / VARIABLES = a1 to a100
   / OUTFILE='C:\GUMSCORE\COMPS\apar.sav'.
END MATRIX.
MATRIX.
GET d / VARIABLES d.
COMPUTE di=transpos(d).
MSAVE di 
   / TYPE=MEAN
   / VARIABLES = d1 to d100
   / OUTFILE='C:\GUMSCORE\COMPS\dpar.sav'.
END MATRIX.
MATRIX.
GET st_0 / VARIABLES t_0.
COMPUTE sti_0 = transpos(st_0).
MSAVE sti_0 
   / TYPE=MEAN
   / VARIABLES = st_0_i1 to st_0_i100
   / OUTFILE='C:\GUMSCORE\COMPS\st_0par.sav'.
END MATRIX.
MATRIX.
GET st_1 / VARIABLES st_1.
COMPUTE sti_1 = transpos(st_1).
MSAVE sti_1 
   / TYPE=MEAN
   / VARIABLES = st_1_i1 to st_1_i100
   / OUTFILE='C:\GUMSCORE\COMPS\st_1par.sav'.
END MATRIX.
MATRIX.
GET st_2 / VARIABLES st_2.
COMPUTE sti_2 = transpos(st_2).
MSAVE sti_2
   / TYPE=MEAN
   / VARIABLES = st_2_i1 to st_2_i100
   / OUTFILE='C:\GUMSCORE\COMPS\st_2par.sav'.
END MATRIX.
MATRIX.
GET st_3 / VARIABLES st_3.
COMPUTE sti_3 = transpos(st_3).
MSAVE sti_3
   / TYPE=MEAN
   / VARIABLES = st_3_i1 to st_3_i100
   / OUTFILE='C:\GUMSCORE\COMPS\st_3par.sav'.
END MATRIX.
MATRIX.
GET st_4 / VARIABLES st_4.
COMPUTE sti_4 = transpos(st_4).
MSAVE sti_4
   / TYPE=MEAN
   / VARIABLES = st_4_i1 to st_4_i100
   / OUTFILE='C:\GUMSCORE\COMPS\st_4par.sav'.
END MATRIX.
MATRIX.
GET st_5 / VARIABLES st_5.
COMPUTE sti_5 = transpos(st_5).
MSAVE sti_5
   / TYPE=MEAN
   / VARIABLES = st_5_i1 to st_5_i100
   / OUTFILE='C:\GUMSCORE\COMPS\st_5par.sav'.
END MATRIX.
MATRIX.
GET st_6 / VARIABLES st_6.
COMPUTE sti_6 = transpos(st_6).
MSAVE sti_6
   / TYPE=MEAN
   / VARIABLES = st_6_i1 to st_6_i100
   / OUTFILE='C:\GUMSCORE\COMPS\st_6par.sav'.
END MATRIX.
MATRIX.
GET st_7 / VARIABLES st_7.
COMPUTE sti_7 = transpos(st_7).
MSAVE sti_7
   / TYPE=MEAN
   / VARIABLES = st_7_i1 to st_7_i100
   / OUTFILE='C:\GUMSCORE\COMPS\st_7par.sav'.
END MATRIX.
MATRIX.
GET st_8 / VARIABLES st_8.
COMPUTE sti_8 = transpos(st_8).
MSAVE sti_8
   / TYPE=MEAN
   / VARIABLES = st_8_i1 to st_8_i100
   / OUTFILE='C:\GUMSCORE\COMPS\st_8par.sav'.
END MATRIX.

GET FILE='C:\GUMSCORE\COMPS\apar.sav'.
EXECUTE.
MATCH FILES /FILE=*
  /FILE='C:\GUMSCORE\COMPS\dpar.sav'
  /RENAME (ROWTYPE_ VARNAME_ = x0 x11) 
  /DROP= x0 x11.
EXECUTE.
MATCH FILES /FILE=*
  /FILE='C:\GUMSCORE\COMPS\st_0par.sav'
  /RENAME (ROWTYPE_ VARNAME_ = x0 x11) 
  /DROP= x0 x11.
EXECUTE.
MATCH FILES /FILE=*
  /FILE='C:\GUMSCORE\COMPS\st_1par.sav'
  /RENAME (ROWTYPE_ VARNAME_ = x0 x11) 
  /DROP= x0 x11.
EXECUTE.
MATCH FILES /FILE=*
  /FILE='C:\GUMSCORE\COMPS\st_2par.sav'
  /RENAME (ROWTYPE_ VARNAME_ = x0 x11) 
  /DROP= x0 x11.
EXECUTE.
MATCH FILES /FILE=*
  /FILE='C:\GUMSCORE\COMPS\st_3par.sav'
  /RENAME (ROWTYPE_ VARNAME_ = x0 x11) 
  /DROP= x0 x11.
EXECUTE.
MATCH FILES /FILE=*
  /FILE='C:\GUMSCORE\COMPS\st_4par.sav'
  /RENAME (ROWTYPE_ VARNAME_ = x0 x11) 
  /DROP= x0 x11.
EXECUTE.
MATCH FILES /FILE=*
  /FILE='C:\GUMSCORE\COMPS\st_5par.sav'
  /RENAME (ROWTYPE_ VARNAME_ = x0 x11) 
  /DROP= x0 x11.
EXECUTE.
MATCH FILES /FILE=*
  /FILE='C:\GUMSCORE\COMPS\st_6par.sav'
  /RENAME (ROWTYPE_ VARNAME_ = x0 x11) 
  /DROP= x0 x11.
EXECUTE.
MATCH FILES /FILE=*
  /FILE='C:\GUMSCORE\COMPS\st_7par.sav'
  /RENAME (ROWTYPE_ VARNAME_ = x0 x11) 
  /DROP= x0 x11.
EXECUTE.
MATCH FILES /FILE=*
  /FILE='C:\GUMSCORE\COMPS\st_8par.sav'
  /RENAME (ROWTYPE_ VARNAME_ = x0 x11) 
  /DROP= x0 x11.
EXECUTE.
SAVE OUTFILE='C:\GUMSCORE\COMPS\ALLPARS.SAV'.
EXECUTE.

GET FILE='C:\GUMSCORE\COMPS\ALLPARS.SAV'.
EXECUTE.
COMPUTE PARNUM=1.
EXECUTE.
RECODE PARNUM (1=30) INTO #NUMROWS.
LOOP COPY=1 TO #NUMROWS.
XSAVE OUTFILE='C:\GUMSCORE\COMPS\ALLPARS_ALL.SAV' / KEEP=COPY ALL.
END LOOP.
EXECUTE.
GET FILE='C:\GUMSCORE\COMPS\ALLPARS_ALL.SAV'.
EXECUTE.
MATCH FILES /FILE=*
  /FILE='C:\GUMSCORE\COMPS\QUADRATURES.sav'.
EXECUTE.
SAVE OUTFILE='C:\GUMSCORE\COMPS\ALLPARS_ALL.SAV'.
EXECUTE.
COMPUTE Af=PDF.NORMAL(Quad,0,1).
EXECUTE.

COMPUTE p=!p.
EXECUTE.
COMPUTE NUMPEOPLE=1.
EXECUTE.
RECODE COPY (1=!p) INTO #NUMPEOPLE.
LOOP COPY=1 TO #NUMPEOPLE.
XSAVE OUTFILE='C:\GUMSCORE\COMPS\ALLPARS_READYFORPEOPLE.SAV' / KEEP=COPY ALL.
END LOOP.
EXECUTE.
GET FILE='C:\GUMSCORE\COMPS\ALLPARS_READYFORPEOPLE.SAV'.
EXECUTE.
SORT CASES BY COPY(A).
SAVE OUTFILE='C:\GUMSCORE\COMPS\ALLPARS_READYFORPEOPLE.SAV' / KEEP=COPY ALL.
EXECUTE.
GET FILE='C:\GUMSCORE\Resp_File.SAV'.
EXECUTE.
COMPUTE CASENUM=$CASENUM.
EXECUTE.
RECODE CASENUM (1=30) INTO #NUMQUADS.
LOOP COPY=1 TO #NUMQUADS.
XSAVE OUTFILE='C:\GUMSCORE\COMPS\Resp_File_MAKE.SAV' / KEEP=COPY ALL.
END LOOP.
EXECUTE.
GET FILE='C:\GUMSCORE\COMPS\Resp_File_make.SAV'.
EXECUTE.
SAVE OUTFILE='C:\GUMSCORE\COMPS\Resp_File_make.SAV'.
EXECUTE.
MATCH FILES /FILE=*
  /FILE='C:\GUMSCORE\COMPS\ALLPARS_READYFORPEOPLE.SAV'.
EXECUTE.
SAVE OUTFILE='C:\GUMSCORE\COMPS\ALL_FORCOMP.SAV'.
EXECUTE.
COMPUTE M=2*!C+1.
EXECUTE.
COMPUTE W0=0.
COMPUTE W1=1.
COMPUTE W2=2.
COMPUTE W3=3.
COMPUTE W4=4.
COMPUTE W5=5.
COMPUTE W6=6.
COMPUTE W7=7.
COMPUTE W8=8.
RECODE case to casenum (-9=SYSMIS).
EXECUTE.
SAVE OUTFILE='C:\GUMSCORE\COMPS\ALL_FORCOMP.SAV'.
EXECUTE.

GET FILE='C:\GUMSCORE\COMPS\ALL_FORCOMP.SAV'.
EXECUTE.
SAVE OUTFILE='C:\GUMSCORE\COMPS\a matrix.SAV'
  /KEEP=a1 to d1
  /COMPRESSED.
EXECUTE.
GET FILE='C:\GUMSCORE\COMPS\a matrix.SAV'.
EXECUTE.
DELETE VARIABLES d1.
SAVE OUTFILE='C:\GUMSCORE\COMPS\a matrix.SAV'.
EXECUTE.

GET FILE='C:\GUMSCORE\COMPS\ALL_FORCOMP.SAV'.
EXECUTE.
SAVE OUTFILE='C:\GUMSCORE\COMPS\d matrix.SAV'
  /KEEP=d1 to st_0_i1
  /COMPRESSED.
EXECUTE.
GET FILE='C:\GUMSCORE\COMPS\d matrix.SAV'.
EXECUTE.
DELETE VARIABLES st_0_i1.
SAVE OUTFILE='C:\GUMSCORE\COMPS\d matrix.SAV'.
EXECUTE.

GET FILE='C:\GUMSCORE\COMPS\ALL_FORCOMP.SAV'.
EXECUTE.
SAVE OUTFILE='C:\GUMSCORE\COMPS\st matrix.SAV'
  /KEEP=st_0_i1 to PARNUM
  /COMPRESSED.
EXECUTE.
GET FILE='C:\GUMSCORE\COMPS\st matrix.SAV'.
EXECUTE.
DELETE VARIABLES PARNUM.
SAVE OUTFILE='C:\GUMSCORE\COMPS\st matrix.SAV'.
EXECUTE.



GET FILE='C:\GUMSCORE\COMPS\ALL_FORCOMP.SAV'.
EXECUTE.
LOOP k=1 TO !i.
VECTOR a=a1 to d1.
VECTOR d=d1 to st_0_i1.
VECTOR st_0_i=st_0_i1 to st_1_i1.
VECTOR st_1_i=st_1_i1 to st_2_i1.
VECTOR st_2_i=st_2_i1 to st_3_i1.
VECTOR st_3_i=st_3_i1 to st_4_i1.
VECTOR st_4_i=st_4_i1 to st_5_i1.
VECTOR st_5_i=st_5_i1 to st_6_i1.
VECTOR st_6_i=st_6_i1 to st_7_i1.
VECTOR st_7_i=st_7_i1 to st_8_i1.
VECTOR st_8_i=st_8_i1 to st_9_i1.
ELSE IF k=!C.
VECTOR st_(k)_i=st_(k)_i1 to parnum.
END IF.
END LOOP.
END LOOP.
LOOP k=1 TO !i.
VECTOR denom_O0(!i).
VECTOR denom_O1(!i).
VECTOR denom_O2(!i).
VECTOR denom_O3(!i).
VECTOR denom_O4(!i).
VECTOR denom_O5(!i).
VECTOR denom_O6(!i).
VECTOR denom_O7(!i).
VECTOR denom_O8(!i).
COMPUTE denom_O0(k)=exp(a(k)*(W0*(QUAD-d(k))-st_0_i(k)))+exp(a(k)*((M-W0)*(QUAD-d(k))-st_0_i(k))).
COMPUTE denom_O1(k)=exp(a(k)*(W1*(QUAD-d(k))-st_1_i(k)))+exp(a(k)*((M-W1)*(QUAD-d(k))-st_1_i(k))).
COMPUTE denom_O2(k)=exp(a(k)*(W2*(QUAD-d(k))-st_2_i(k)))+exp(a(k)*((M-W2)*(QUAD-d(k))-st_2_i(k))).
COMPUTE denom_O3(k)=exp(a(k)*(W3*(QUAD-d(k))-st_3_i(k)))+exp(a(k)*((M-W3)*(QUAD-d(k))-st_3_i(k))).
COMPUTE denom_O4(k)=exp(a(k)*(W4*(QUAD-d(k))-st_4_i(k)))+exp(a(k)*((M-W4)*(QUAD-d(k))-st_4_i(k))).
COMPUTE denom_O5(k)=exp(a(k)*(W5*(QUAD-d(k))-st_5_i(k)))+exp(a(k)*((M-W5)*(QUAD-d(k))-st_5_i(k))).
COMPUTE denom_O6(k)=exp(a(k)*(W6*(QUAD-d(k))-st_6_i(k)))+exp(a(k)*((M-W6)*(QUAD-d(k))-st_6_i(k))).
COMPUTE denom_O7(k)=exp(a(k)*(W7*(QUAD-d(k))-st_7_i(k)))+exp(a(k)*((M-W7)*(QUAD-d(k))-st_7_i(k))).
COMPUTE denom_O8(k)=exp(a(k)*(W8*(QUAD-d(k))-st_8_i(k)))+exp(a(k)*((M-W8)*(QUAD-d(k))-st_8_i(k))).
VECTOR denom_i(!i).
COMPUTE x=$sysmis.
COMPUTE denom_i(k)=sum(denom_O0(k),denom_O1(k),denom_O2(k),denom_O3(k),denom_O4(k),denom_O5(k),denom_O6(k),denom_O7(k),denom_O8(k)).

VECTOR i_=i_1 to CASENUM.

VECTOR pi_(!i).
DO IF i_(k)=0.
COMPUTE pi_(k)=denom_O0(k)/denom_i(k).
END IF.
DO IF i_(k)=1.
COMPUTE pi_(k)=denom_O1(k)/denom_i(k).
END IF.
DO IF i_(k)=2.
COMPUTE pi_(k)=denom_O2(k)/denom_i(k).
END IF.
DO IF i_(k)=3.
COMPUTE pi_(k)=denom_O3(k)/denom_i(k).
END IF.
DO IF i_(k)=4.
COMPUTE pi_(k)=denom_O4(k)/denom_i(k).
END IF.
DO IF i_(k)=5.
COMPUTE pi_(k)=denom_O5(k)/denom_i(k).
END IF.
DO IF i_(k)=6.
COMPUTE pi_(k)=denom_O6(k)/denom_i(k).
END IF.
DO IF i_(k)=7.
COMPUTE pi_(k)=denom_O7(k)/denom_i(k).
END IF.
DO IF i_(k)=8.
COMPUTE pi_(k)=denom_O8(k)/denom_i(k).
END IF.

COMPUTE z=$sysmis.
VECTOR prod_(!i).
COMPUTE prod_1=pi_1.
DO IF k>1.
END IF.
COMPUTE prod_(k)=prod_(k-1)*pi_(k).
COMPUTE LVf=prod_(k).
END LOOP.
EXECUTE.
COMPUTE theta_numer=Quad*Af*LVf.
COMPUTE thetaANDse_denom=Af*LVf.
EXECUTE.

AGGREGATE
  /OUTFILE=* MODE=ADDVARIABLES
  /BREAK=case
  /theta_numer_sum=SUM(theta_numer) 
  /thetaANDse_denom_sum=SUM(thetaANDse_denom).
COMPUTE theta=theta_numer_sum/thetaANDse_denom_sum.
EXECUTE.
COMPUTE se_numer=((Quad-theta)**2)*Af*LVf.
EXECUTE.

AGGREGATE
  /OUTFILE=* MODE=ADDVARIABLES
  /BREAK=case
  /se_numer_sum=SUM(se_numer).

COMPUTE se_theta=SQRT(se_numer_sum/thetaANDse_denom_sum).
EXECUTE.
SAVE OUTFILE='C:\GUMSCORE\COMPS\ALL_FORCOMP.SAV'.
EXECUTE.
GET FILE='C:\GUMSCORE\COMPS\ALL_FORCOMP.SAV'.
EXECUTE.

SORT CASES BY case.
AGGREGATE
  /OUTFILE='C:\GUMSCORE\RESULTS_Theta_SE.sav'
  /PRESORTED
  /BREAK=case
  /theta_mean=MEAN(theta) 
  /se_theta_mean=MEAN(se_theta).

GET FILE='C:\GUMSCORE\RESULTS_Theta_SE.sav'.
EXECUTE.
MATCH FILES /FILE=*
  /FILE='C:\GUMSCORE\Resp_File.SAV'
  /RENAME (case = d0) 
  /DROP= d0.

RENAME VARIABLES (theta_mean se_theta_mean = theta_EAP se_theta_EAP.
EXECUTE.
SAVE OUTFILE='C:\GUMSCORE\RESULTS_Theta_SE.sav'.
EXECUTE.
ERASE FILE 'C:\GUMSCORE\COMPS\a matrix.sav'.
ERASE FILE 'C:\GUMSCORE\COMPS\d matrix.sav'.
ERASE FILE 'C:\GUMSCORE\COMPS\st_0par.sav'.
ERASE FILE 'C:\GUMSCORE\COMPS\st_1par.sav'.
ERASE FILE 'C:\GUMSCORE\COMPS\st_2par.sav'.
ERASE FILE 'C:\GUMSCORE\COMPS\st_3par.sav'.
ERASE FILE 'C:\GUMSCORE\COMPS\st_4par.sav'.
ERASE FILE 'C:\GUMSCORE\COMPS\st_5par.sav'.
ERASE FILE 'C:\GUMSCORE\COMPS\st_6par.sav'.
ERASE FILE 'C:\GUMSCORE\COMPS\st_7par.sav'.
ERASE FILE 'C:\GUMSCORE\COMPS\st_8par.sav'.
ERASE FILE 'C:\GUMSCORE\COMPS\apar.sav'.
ERASE FILE 'C:\GUMSCORE\COMPS\dpar.sav'.
ERASE FILE 'C:\GUMSCORE\COMPS\ALLPARS_READYFORPEOPLE.sav'.
ERASE FILE 'C:\GUMSCORE\COMPS\ALLPARS.sav'.
ERASE FILE 'C:\GUMSCORE\COMPS\st matrix.sav'.
ERASE FILE 'C:\GUMSCORE\COMPS\Resp_File_make.sav'.
ERASE FILE 'C:\GUMSCORE\COMPS\ALLPARS_ALL.sav'.
ERASE FILE 'C:\GUMSCORE\COMPS\ALL_FORCOMP.sav'.
EXECUTE.
GET FILE='C:\GUMSCORE\RESULTS_Theta_SE.sav'.
EXECUTE.
!ENDDEFINE.